#ifndef __DSPLIB_H__
#define __DSPLIB_H__

#include <vstypes.h>

/** Bi-quad IIR coefficients in 2.14 format. */
struct DSP_IIR2COEFF16 {
    s_int16 b0,b1,b2,a1,a2;
};

/** Calculates bi-quad IIR for one sample. 26 cycles.
 */
__near auto s_int16 DspIir2Lq(register __a0 s_int16 val,
			      register __i0 __near const __mem_y struct DSP_IIR2COEFF16 *baCoeff,
			      register __i2 __near s_int16 mem[5]);
/** Calculates bi-quad IIR for a block of samples. 27+10*n cycles.
 */
__near auto void DspIir2nLq(register __i1 __near s_int16 *buffer,
			    register __i0 __near const __mem_y struct DSP_IIR2COEFF16 *baCoeff,
			    register __i2 __near s_int16 mem[5], register __a1 s_int16 n);

/** Calculates a bi-quad IIR for one sample. (20200312)
    IIR coefficients in 2.14 format.

    Execution time 42 cycles.

    val   - the value
    coeff - a pointer to coefficients (5 x s_int16)
    mem   - a pointer to memory (4 x s_int32).

    returns the filtered value
 */
auto s_int16 DspIir2Mq(register __a0 s_int16 val,
		    register __i0 const __mem_y struct DSP_IIR2COEFF16 *coeff,
		    register __i2 s_int32 mem[4]);

/** Calculates numFilters x bi-quad IIR for numSample samples. (20200312)
    IIR coefficients in 2.14 format. Calculating multiple filters in
    one call produces slightly more accurate results than using separate
    calls for each filter.

    Execution time 14+numSamples*(8+numFilters*28) cycles.

    val   - a pointer to numSamples of s_int16 samples to process
    numSamples - the number of samples to process
    numFilter  - the number of filters to perform for each sample
    coeff - a pointer to numFilters of coefficients (each 5 x s_int16)
    mem   - a pointer to numFilters of memory (each 4 x s_int32).
 */
auto void DspIir2MqNM(register __i1 s_int16 *val,
		      register __a0 s_int16 numSamples,
		      register __a1 s_int16 numFilters,
		      register __i0 const __mem_y struct DSP_IIR2COEFF16 *coeff,
		      register __i2 s_int32 mem[4]);

/** Calculates numFilters x bi-quad IIR for numSample samples. (20200312)
    IIR coefficients in 2.14 format. Calculating multiple filters in
    one call produces slightly more accurate results than using separate
    calls for each filter.

    Execution time 14+numSamples*(8+numFilters*28) cycles.

    val   - a pointer to numSamples of s_int32 samples to process
    numSamples - the number of samples to process
    numFilter  - the number of filters to perform for each sample
    coeff - a pointer to numFilters of coefficients (each 5 x s_int16)
    mem   - a pointer to numFilters of memory (each 4 x s_int32).
 */
auto void DspIir2Mq32NM(register __i1 s_int32 *val,
			register __a0 s_int16 numSamples,
			register __a1 s_int16 numFilters,
			register __i0 const __mem_y struct DSP_IIR2COEFF16 *coeff,
			register __i2 s_int32 mem[4]);


/** Calculates 64-bit multiplication result. 12 cycles.
 */
auto void Mul64(register __reg_a s_int32 a, register __reg_d s_int32 d,
		register __i0 s_int32 reslohi[2]);

/** Calculates sine from phase. (244+ cycles)
    \param phase - fractional value, range [-1.0 1.0) is [-180 180) degrees
    \return value in range of [-1.0 1.0)
    \bug Added saturation 20210219.
 */
s_int32 frsine32(register __reg_c s_int32 phase);

/*
  CRC16 functions for the CRC used for the sector data of MMC/SD cards.
 */
extern const u_int16 sdCrcTable[256];
auto u_int16 SdCrcBlock(register __i2 u_int16 *block, register __a0 s_int16 words);
auto u_int16 SdCrcBlockChained(register __i2 u_int16 *block, register __a0 u_int16 startCrc, register __a1 s_int16 words);

/*
  CRC32 - table and functions for 32-bit CRC, which is used e.g. in Ogg.
 */
extern const u_int32 __mem_y oggCrcTable[256];
//A is 32-bit variable, b is the new byte
#define UPDCRC32(a,b) (oggCrcTable[(((s_int32)(a)>>24) ^ (b)) & 0xff] ^ ((a)<<8))
/* u_int32 crc32 = 0;
   crc32 = UPDCRC32(crc32, *s++);
*/
/* Calculates a CRC-32 from the bytes packed into the words. i.e. only
   even number of bytes can be calculated using this routine. */
auto u_int32 OggCrcBlockChained(register __i2 u_int16 *block, register __c0 s_int16 words, register __reg_a u_int32 startCrc);

/**
   Calculates phase from a complex value. Note the reversed parameter order
   compared to atan2.
   35.5 clock cycles, max error +-0.044%.
   \return phase - fractional value, range [-32768 32767] is [-180 180) degrees
 */
auto s_int16 Phase(register __c0 s_int16 re, register __c1 s_int16 im);


/**
   Converts a vector of IEEE floats in little-endian format to 32-bit
   signed format with saturation. from and to may be the same pointer.
 */
void IEEEto32(register __i2 void *from, register __i3 s_int32 *to,
	      register __a0 s_int16 samples);
/**
   Converts a vector of IEEE floats in little-endian format to 32-bit
   signed format with saturation. from and to may be the same pointer.
 */
void IEEEto32y(register __i2 __mem_y void *from, register __i3 __mem_y s_int32 *to,
	      register __a0 s_int16 samples);

/**
   FadeVector32Stride - scale 32-bit data.
   p     - data to scale by gain
   skipWords - number of words to skip, usually sizeof(s_int32)
   beg   - starting gain
   delta - gain change per element
   returns the end gain
 */
register __c0 f_int16 FadeVector32Stride(register __i0 s_int32 *p, register __i1 s_int16 skipWords, register __c0 f_int16 beg, register __c1 f_int16 delta, register __a0 s_int16 samples);


/** 20200408

   DSPFIR32 use example:

#define FIR_SIZE 123
   __align s_int32 firdelay[FIR_SIZE];
  const __mem_y coefficients[FIR_SIZE] = {32767, 0};
  struct DSPFIR32 myfir;
  DspFir32Init(&myfir, firdelay, coefficients, FIR_SIZE, 1);
  ...
  DspFir32X(&myfir, samples, numSamples);
**/

struct DSPFIR32 {
    __align s_int32 *delay;
    __mem_y const s_int16 *coeff;
    s_int16 fSize; /*Filter size, both delay and coeffs the same #elements!*/
    __align s_int32 *phase; /* location for the next sample in */
    s_int16 shift;
    u_int16 modF; /*Modulo forward, includes size info of delay ring buffer */
};

/** Performs the initialization of the DSPFIR32 structure. **/
void DspFir32Init(register __i1 struct DSPFIR32 *fir,
		  __align s_int32 *delay,
		  __mem_y const s_int16 *coeff,
		  s_int16 fSize,
		  s_int16 shift);
/**
   Performs in-place filtering of len samples at samples according to
   coefficients, delay line, and other information from fir.
 */
void DspFir32X(register __i1 struct DSPFIR32 *fir,
	       register __i3 s_int32 *samples, register __a0 s_int16 len);
/**
   Performs in-place filtering of len samples at samples in Y memory
   according to coefficients, delay line, and other information from fir.
 */
void DspFir32Y(register __i1 struct DSPFIR32 *fir,
	       register __i3 __mem_y s_int32 *samples, register __a0 s_int16 len);
/**
   Performs filtering of len samples at insamples according to
   coefficients, delay line, and other information from fir and
   writes the resulting samples to outsamples in Y memory.
 */
void DspFir32XtoY(register __i1 struct DSPFIR32 *fir,
		  register __i2 s_int32 *insamples,
		  register __i3 __mem_y s_int32 *outsamples,
		  register __a0 s_int16 len);
/**
   Performs filtering of len samples at insamples according to
   coefficients, delay line, and other information from fir and
   writes the resulting samples to outsamples in X memory.
   insamples and outsamples can be the same buffer.
 */
void DspFir32XtoX(register __i1 struct DSPFIR32 *fir,
		  register __i2 s_int32 *insamples,
		  register __i3 s_int32 *outsamples,
		  register __a0 s_int16 len);
/**
   Performs filtering of len samples at insamples in Y memory according to
   coefficients, delay line, and other information from fir and
   writes the resulting samples to outsamples in X memory.
 */
void DspFir32YtoX(register __i1 struct DSPFIR32 *fir,
		  register __i2 __mem_y s_int32 *insamples,
		  register __i3 s_int32 *outsamples,
		  register __a0 s_int16 len);
/**
   Performs filtering of len samples at insamples in Y memory according to
   coefficients, delay line, and other information from fir and
   writes the resulting samples to outsamples in Y memory.
   insamples and outsamples can be the same buffer.
 */
void DspFir32YtoY(register __i1 struct DSPFIR32 *fir,
		  register __i2 __mem_y s_int32 *insamples,
		  register __i3 __mem_y s_int32 *outsamples,
		  register __a0 s_int16 len);



#endif /* !__DSPLIB_H__ */
